<?php

declare(strict_types = 1);

namespace Drupal\guest_suite;

use Drupal\Core\CronInterface;
use Drupal\Core\Queue\QueueFactory;

/**
 * Default cron implementation.
 */
class Cron implements CronInterface {

  /**
   * The queue factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The review manager.
   *
   * @var \Drupal\guest_suite\ApiConsumer
   */
  protected $apiConsumer;

  /**
   * Constructs a new Cron object.
   *
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   The queue factory.
   * @param \Drupal\guest_suite\ApiConsumer $api_consumer
   *   The Guest Suite API consumer.
   */
  public function __construct(QueueFactory $queue_factory, ApiConsumer $api_consumer) {
    $this->queueFactory = $queue_factory;
    $this->apiConsumer = $api_consumer;
  }

  /**
   * {@inheritdoc}
   */
  public function run(): void {
    // Module config can not be injected because we need no cache
    // if the config was modified between 2 CRON runs.
    if (!\Drupal::config('guest_suite.settings')->get('cron_import')) {
      return;
    }

    // Prevents duplicate queue items from being created
    $fetch_queue = $this->queueFactory->get('guest_suite_review_fetcher');
    $import_queue = $this->queueFactory->get('guest_suite_review_importer');
    if ($fetch_queue->numberOfItems() > 0 || $import_queue->numberOfItems() > 0) {
      return;
    }

    $statistics = $this->apiConsumer->getStatistics();
    if (empty($statistics['total'])) {
      return; // Finish early.
    }

    $page_total = ceil($statistics['total'] / 100);
    for ($page_index = 0; $page_index < $page_total; $page_index++) {
      $fetch_queue->createItem($page_index);
    }
  }

}
